//+------------------------------------------------------------------+
//|                                                  MACD Sample.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+

extern int        P_TakeProf              = 50;
extern double     P_Lots                  = 0.1;
extern bool       P_TrailStop             = true;
extern int        P_ZBar                  = 0;
extern double     P_SARStep               = 0.026; 
extern double     P_SARMax                = 0.26; 
extern int        P_GreenLMax             = 68; //upper long entry limit
extern int        P_GreenLMin             = 50; //lower long entry limit
extern int        P_GreenSMax             = 50; //upper short entry limit
extern int        P_GreenSMin             = 32; //lower short entry limit
extern int        P_STDMin                = 7;
extern int        P_ATRMin                = 9;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int cnt, ticket, total;
// initial data checks
// it is important to make sure that the expert works with a normal
// chart and the user did not make any mistakes setting external 
// variables (Lots, StopLoss, TakeProfit, 
// TrailingStop) in our case, we check TakeProfit
// on a chart of less than 100 bars
   if(Bars<100)
     {
      Print("bars less than 100");
      return(0);  
     }
   if(P_TakeProf<10)
     {
      Print("TakeProfit less than 10");
      return(0);  // check TakeProfit
     }
// to simplify the coding and speed up access
// data are put into internal variables


//========================== Traders Dynamic Index ====================
   double RSI[];
   ArrayResize(RSI,35);
   ArraySetAsSeries(RSI,true);
   for(int i=P_ZBar; i<=35; i++)
      {
           RSI[i] = (iRSI(NULL,0,13,0,i)); 
      }
       double Green0 = iMAOnArray(RSI,0,2,0,MODE_SMA,P_ZBar);
       double Green1 = iMAOnArray(RSI,0,2,0,MODE_SMA,(P_ZBar+1));
//Print("Green0 = ",Green0);

       double Red0   = iMAOnArray(RSI,0,7,0,MODE_SMA,P_ZBar);
       double Red1   = iMAOnArray(RSI,0,7,0,MODE_SMA,(P_ZBar+1));
//Print("Red0 = ",Red0);

      double Yellow0 = iMAOnArray(RSI,0,34,0,MODE_SMA,P_ZBar);
      double Yellow1 = iMAOnArray(RSI,0,34,0,MODE_SMA,(P_ZBar+1));
//Print("Yellow0 = ",Yellow0);

      double BlueUpp0 = iBandsOnArray(RSI,0,34,1.6185,0,MODE_UPPER,P_ZBar);  
      double BlueUpp1 = iBandsOnArray(RSI,0,34,1.6185,0,MODE_UPPER,(P_ZBar+1));  
      double BlueLow0 = iBandsOnArray(RSI,0,34,1.6185,0,MODE_LOWER,P_ZBar);  
      double BlueLow1 = iBandsOnArray(RSI,0,34,1.6185,0,MODE_LOWER,(P_ZBar+1));
//Print("BlueUpp0 = ",BlueUpp0);
//Print("BlueLow0 = ",BlueLow0);

//========================== Standard Deviation on RSI ====================

       double STDRSI =  iStdDevOnArray(RSI,0,13,0,MODE_EMA,P_ZBar);
Print("STDRSI = ",STDRSI);

//========================== Price Action Channel ====================

    double PACUpp0 = iMA(NULL,0,5,0,MODE_SMMA,PRICE_HIGH,P_ZBar);
    double PACUpp1 = iMA(NULL,0,5,0,MODE_SMMA,PRICE_HIGH,(P_ZBar+1));
//Print("PACUpp0 = ",PACUpp0);

    double PACLow0  = iMA(NULL,0,5,0,MODE_SMMA,PRICE_LOW,P_ZBar);
    double PACLow1  = iMA(NULL,0,5,0,MODE_SMMA,PRICE_LOW,(P_ZBar+1));
//Print("PACLow0 = ",PACLow0);

//========================== Heiken Ashi ====================
//could be replaced by 2 period LWMA on Typical or Weighted Price

    double haClose0 = (Open[P_ZBar]+High[P_ZBar]+Low[P_ZBar]+Close[P_ZBar])*(0.25);
    double haClose1 = (Open[(P_ZBar+1)]+High[(P_ZBar+1)]+Low[(P_ZBar+1)]+Close[(P_ZBar+1)])*(0.25);
//Print("haClose0 = ",haClose0);

//========================== Parabolic SAR and Moving Average centreline ===============

    double PSAR0       = iSAR(NULL,0,P_SARStep,P_SARMax,0);    
    double PSAR1       = iSAR(NULL,0,P_SARStep,P_SARMax,1); 
    double MAV0        = iMA (NULL,0,3,0,MODE_LWMA,PRICE_WEIGHTED,P_ZBar);
    double MAV1        = iMA (NULL,0,3,0,MODE_LWMA,PRICE_WEIGHTED,(P_ZBar+1));
//Print("MAV0 = ",MAV0);

//====================== Average True Range on Price ==========================
    double ATR0 = iATR(NULL,0,13,0);
    
   total=OrdersTotal();
   if(total<1) 
     {
      // no opened orders identified
      if(AccountFreeMargin()<(1000*P_Lots))
        {
         Print("We have no money. Free Margin = ", AccountFreeMargin());
         return(0);  
        }
      // check for long position (BUY) possibility
      if(
//========================== Heiken Ashi ====================
                  haClose0 >  PACUpp0
               && haClose0 >  haClose1
//========================== Price Action Channel ====================
               && PACUpp0  >  PACUpp1
//========================== Traders Dynamic Index ====================
               && Green0   >  Green1
               && Green0   >  P_GreenLMin
               && Green0   <  P_GreenLMax
               && Green0   >= Red0
               && Red0     >= Red1
               && Green0   >= Yellow0
               && Yellow0  >= Yellow1
//========================== Standard Deviation on RSI ====================
               && STDRSI   >  P_STDMin
//========================== Parabolic SAR and Moving Average centreline ===============
               && PSAR0    <  MAV0
               && ATR0     >  (P_ATRMin*Point)
        )
        
        {
         ticket=OrderSend(Symbol(),OP_BUY,P_Lots,Ask,3,0,Ask+P_TakeProf*Point,"SynergyPSAR",16384,0,Green);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("BUY order opened : ",OrderOpenPrice());
           }
         else Print("Error opening BUY order : ",GetLastError()); 
         return(0); 
        }
      // check for short position (SELL) possibility
      if(
//========================== Heiken Ashi ====================
                  haClose0 <  PACLow0
               && haClose0 <  haClose1
//========================== Price Action Channel ====================
               && PACLow0  <  PACLow1
//========================== Traders Dynamic Index ====================
               && Green0   <  Green1
               && Green0   >  P_GreenSMin
               && Green0   <  P_GreenSMax
               && Green0   <= Red0
               && Red0     <= Red1
               && Green0   <= Yellow0
               && Yellow0  <= Yellow1
//========================== Standard Deviation on RSI ====================
               && STDRSI   >  P_STDMin
//========================== Parabolic SAR and Moving Average centreline ===============
               && PSAR0    >  MAV0
//====================== Average True Range on Price ==========================
               && ATR0     >  (P_ATRMin*Point)
         )
        {
         ticket=OrderSend(Symbol(),OP_SELL,P_Lots,Bid,3,0,Bid-P_TakeProf*Point,"SynergyPSAR",16384,0,Red);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("SELL order opened : ",OrderOpenPrice());
           }
         else Print("Error opening SELL order : ",GetLastError()); 
         return(0); 
        }
      return(0);
     }
   // it is important to enter the market correctly, 
   // but it is more important to exit it correctly...   
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL &&   // check for opened position 
         OrderSymbol()==Symbol())  // check for symbol
        {
         if(OrderType()==OP_BUY)   // long position is opened
           {
            // should it be closed?
            if(
                    PSAR1 <  MAV1
                 && PSAR0 >  MAV0
//                && Green0  <= Green0
                ||(
                 (   haClose0 < haClose1)
                &&(
                 (   haClose0 <  PACUpp0)
                 ||
                 (   Green1 >  P_GreenLMax
                  && Green0 <= P_GreenLMax)
                 ||
                 (   Green1   >  Red1
                  && Green0   <= Red0)
                  ||
                   (  Green0 <=  BlueUpp0
                   && Green1  >  BlueUpp1)
                 ))
               )
                {
                 OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet); // close position
                 return(0); // exit
                }
            // check for trailing stop
            if(P_TrailStop==true)  
//              {                 
//               if(Bid-OrderOpenPrice()>Point*TrailingStop)
//                 {
//                  if(OrderStopLoss()<Bid-Point*TrailingStop)
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),PSAR0,OrderTakeProfit(),0,Green);
                     return(0);
                    }
//                 }
//              }
           }
         else // go to short position
           {
            // should it be closed?
            if(
                    PSAR1 >  MAV1
                 && PSAR0 <  MAV0
//                && Green0  >= Green1)
                &&(
                 (   haClose0 > haClose1)
                &&(
                 (   haClose0 >  PACLow0)
                 ||
                   ( Green1 <= P_GreenSMin
                  && Green0 >  P_GreenSMin)
                 ||
                   ( Green1 <  Red1
                  && Green0 >= Red0)
                 ||
                   ( Green0 >= BlueLow0
                  && Green1 <  BlueLow1)
                 ))
               )
              {
               OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet); // close position
               return(0); // exit
              }
            // check for trailing stop
            if(P_TrailStop==true)  
//              {                 
//               if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
//                 {
//                  if((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),PSAR0,OrderTakeProfit(),0,Red);
                     return(0);
                    }
//                 }
//              }
           }
        }
     }
   return(0);
  }
// the end.